@@ -471,8 +471,6 @@ scroll_row_timeout (gpointer data)
 	GdkWindow *window;
 	GdkDeviceManager *device_manager;
 
-	GDK_THREADS_ENTER ();
-
 	priv_data = g_object_get_data (G_OBJECT (tree_view), RB_TREE_DND_STRING);
 	g_return_val_if_fail(priv_data != NULL, TRUE);
 
@@ -487,7 +485,6 @@ scroll_row_timeout (gpointer data)
 	/* see if we are near the edge. */
 	if (x < visible_rect.x && x > visible_rect.x + visible_rect.width)
 	{
-		GDK_THREADS_LEAVE ();
 		priv_data->scroll_timeout = 0;
 		return FALSE;
 	}
@@ -498,7 +495,6 @@ scroll_row_timeout (gpointer data)
 		offset = y - (visible_rect.y + visible_rect.height - 2 * SCROLL_EDGE_SIZE);
 		if (offset < 0) 
 		{
-			GDK_THREADS_LEAVE ();
 			priv_data->scroll_timeout = 0;
 			return FALSE;
 		}
@@ -515,8 +511,6 @@ scroll_row_timeout (gpointer data)
 	if (ABS (vadj_value - value) > 0.0001)
 		remove_select_on_drag_timeout(tree_view);
 
-	GDK_THREADS_LEAVE ();
-
 	return TRUE;
 }
 
@@ -528,8 +522,6 @@ select_on_drag_timeout (gpointer data)
 	GtkTreeSelection *selection;
 	RbTreeDndData *priv_data;
 
-	GDK_THREADS_ENTER ();
-
 	priv_data = g_object_get_data (G_OBJECT (tree_view), RB_TREE_DND_STRING);
 	g_return_val_if_fail(priv_data != NULL, FALSE);
 	g_return_val_if_fail(priv_data->previous_dest_path != NULL, FALSE);
@@ -543,8 +535,6 @@ select_on_drag_timeout (gpointer data)
 	priv_data->select_on_drag_timeout = 0;
 	gtk_tree_path_free(priv_data->previous_dest_path);
 	priv_data->previous_dest_path = NULL;
-
-	GDK_THREADS_LEAVE ();
 	return FALSE;
 }
 
