@@ -400,23 +400,8 @@ purge_useless_threads (gpointer data)
 	return TRUE;
 }
 
-
-static GRecMutex rb_gdk_mutex;
 static gboolean mutex_recurses;
 
-static void
-_threads_enter (void)
-{
-	g_rec_mutex_lock (&rb_gdk_mutex);
-}
-
-static void
-_threads_leave (void)
-{
-	g_rec_mutex_unlock (&rb_gdk_mutex);
-}
-
-
 /**
  * rb_assert_locked: (skip)
  * @mutex: a #GMutex
@@ -443,10 +428,6 @@ rb_threads_init (void)
 
 	g_private_set (&private_is_primary_thread, GUINT_TO_POINTER (1));
 
-	g_rec_mutex_init (&rb_gdk_mutex);
-	gdk_threads_set_lock_functions (_threads_enter, _threads_leave);
-	gdk_threads_init ();
-
 	g_mutex_init (&m);
 	g_mutex_lock (&m);
 	mutex_recurses = g_mutex_trylock (&m);
@@ -1222,6 +1203,14 @@ rb_set_tree_view_column_fixed_width (GtkWidget  *treeview,
 	int max_width = 0;
 	int i = 0;
 
+	/* Take into account the header button width */
+	GtkWidget *widget = gtk_tree_view_column_get_button (column);
+	if (widget != NULL) {
+		GtkRequisition natural_size;
+		gtk_widget_get_preferred_size (widget, NULL, &natural_size);
+		max_width = natural_size.width;
+	}
+
 	while (strings[i] != NULL) {
 		GtkRequisition natural_size;
 		g_object_set (renderer, "text", strings[i], NULL);
@@ -1290,9 +1279,7 @@ do_delayed_apply (GSettings *settings)
 	data = g_object_get_data (G_OBJECT (settings), DELAYED_SYNC_DATA_ITEM);
 	sync_func = g_object_get_data (G_OBJECT (settings), DELAYED_SYNC_FUNC_ITEM);
 	if (sync_func != NULL) {
-		GDK_THREADS_ENTER ();
 		sync_func (settings, data);
-		GDK_THREADS_LEAVE ();
 	}
 
 	g_object_set_data (G_OBJECT (settings), DELAYED_SYNC_ITEM, GUINT_TO_POINTER (0));
