@@ -33,8 +33,6 @@ from gi.repository import GObject, GLib, Gio
 IMAGE_NAMES = ["cover", "album", "albumart", "front", ".folder", "folder"]
 ITEMS_PER_NOTIFICATION = 10
 
-IGNORED_SCHEMES = ('http', 'cdda', 'daap', 'mms')
-
 def file_root (f_name):
 	return os.path.splitext (f_name)[0].lower ()
 
@@ -72,7 +70,7 @@ class LocalSearch:
 					nkey = RB.ExtDBKey.create_storage("album", album)
 					nkey.add_field("artist", artist)
 					uri = parent.resolve_relative_path(f_name).get_uri()
-					print "found album+artist match " + uri
+					print("found album+artist match " + uri)
 					self.store.store_uri(nkey, RB.ExtDBSourceType.USER, uri)
 
 		# if that didn't work, look for the longest shared prefix
@@ -87,16 +85,24 @@ class LocalSearch:
 
 		if match is not None:
 			uri = parent.resolve_relative_path(match).get_uri()
-			print "found prefix match " + uri
+			print("found prefix match " + uri)
 			self.store.store_uri(key, RB.ExtDBSourceType.USER, uri)
 
 		self.callback(self.callback_args)
 
+	def _close_enum_cb(self, fileenum, result, results):
+		try:
+			fileenum.close_finish(result)
+		except Exception as e:
+			print("couldn't close file enumerator: %s" % e)
+		
+
 	def _enum_dir_cb(self, fileenum, result, results):
 		try:
 			files = fileenum.next_files_finish(result)
 			if files is None or len(files) == 0:
-				print "okay, done; got %d files" % len(results)
+				print("okay, done; got %d files" % len(results))
+				fileenum.close_async(GLib.PRIORITY_DEFAULT, None, self._close_enum_cb, None)
 				self.finished(results)
 				return
 
@@ -110,21 +116,23 @@ class LocalSearch:
 					results.append(f.get_name())
 
 			fileenum.next_files_async(ITEMS_PER_NOTIFICATION, GLib.PRIORITY_DEFAULT, None, self._enum_dir_cb, results)
-		except Exception, e:
-			print "okay, probably done: %s" % e
+		except Exception as e:
+			print("okay, probably done: %s" % e)
 			import sys
 			sys.excepthook(*sys.exc_info())
 			self.finished(results)
+			fileenum.close_async(GLib.PRIORITY_DEFAULT, None, self._close_enum_cb, None)
 
 
 	def _enum_children_cb(self, parent, result, data):
 		try:
 			enumfiles = parent.enumerate_children_finish(result)
 			enumfiles.next_files_async(ITEMS_PER_NOTIFICATION, GLib.PRIORITY_DEFAULT, None, self._enum_dir_cb, [])
-		except Exception, e:
-			print "okay, probably done: %s" % e
-			import sys
-			sys.excepthook(*sys.exc_info())
+		except Exception as e:
+			print("okay, probably done: %s" % e)
+			if not isinstance(e, GLib.GError):
+				import sys
+				sys.excepthook(*sys.exc_info())
 			self.callback(self.callback_args)
 
 
@@ -133,15 +141,11 @@ class LocalSearch:
 
 		location = key.get_info("location")
 		if location is None:
-			print "not searching, we don't have a location"
+			print("not searching, we don't have a location")
 			callback(args)
 			return
 
 		self.file = Gio.file_new_for_uri(location)
-		if self.file.get_uri_scheme() in IGNORED_SCHEMES:
-			print 'not searching for local art for %s' % (self.file.get_uri())
-			callback(args)
-			return
 
 		self.album = key.get_field("album")
 		self.artists = key.get_field_values("artist")
@@ -149,6 +153,6 @@ class LocalSearch:
 		self.callback = callback
 		self.callback_args = args
 
-		print 'searching for local art for %s' % (self.file.get_uri())
+		print('searching for local art for %s' % (self.file.get_uri()))
 		parent = self.file.get_parent()
 		enumfiles = parent.enumerate_children_async("standard::content-type,access::can-read,standard::name", 0, 0, None, self._enum_children_cb, None)
