@@ -30,7 +30,6 @@
 #include <string.h>
 #include <unistd.h>
 #include <libsoup/soup.h>
-#include <libsoup/soup-gnome.h>
 #include <json-glib/json-glib.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
@@ -373,7 +372,7 @@ rb_audioscrobbler_radio_source_init (RBAudioscrobblerRadioSource *source)
 
 	source->priv->soup_session =
 		soup_session_async_new_with_options (SOUP_SESSION_ADD_FEATURE_BY_TYPE,
-		                                     SOUP_TYPE_GNOME_FEATURES_2_26,
+		                                     SOUP_TYPE_PROXY_RESOLVER_DEFAULT,
 		                                     NULL);
 }
 
