@@ -23,8 +23,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -306,8 +305,6 @@ mdns_service_added (DMAPMdnsBrowser *browser,
 		   service->port,
 		   service->password_protected);
 
-	GDK_THREADS_ENTER ();
-
 	source = find_source_by_service_name (plugin, service->service_name);
 
 	if (source == NULL) {
@@ -334,8 +331,6 @@ mdns_service_added (DMAPMdnsBrowser *browser,
 			      "password-protected", service->password_protected,
 			      NULL);
 	}
-
-	GDK_THREADS_LEAVE ();
 }
 
 static void
@@ -344,17 +339,12 @@ mdns_service_removed (DMAPMdnsBrowser *browser,
 		      RBDaapPlugin	*plugin)
 {
 	RBSource *source;
-
-	GDK_THREADS_ENTER ();
-
 	source = find_source_by_service_name (plugin, service_name);
 
 	rb_debug ("DAAP source '%s' went away", service_name);
 	if (source != NULL) {
 		g_hash_table_remove (plugin->source_lookup, service_name);
 	}
-
-	GDK_THREADS_LEAVE ();
 }
 
 static void
