@@ -598,8 +598,6 @@ import_complete_cb (RhythmDBImportJob *job, int total, RBGenericPlayerSource *so
 	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 	RBShell *shell;
 
-	GDK_THREADS_ENTER ();
-
 	if (priv->ejecting) {
 		rb_device_source_default_eject (RB_DEVICE_SOURCE (source));
 	} else {
@@ -617,8 +615,6 @@ import_complete_cb (RhythmDBImportJob *job, int total, RBGenericPlayerSource *so
 
 	g_object_unref (priv->import_job);
 	priv->import_job = NULL;
-
-	GDK_THREADS_LEAVE ();
 }
 
 static void
@@ -934,7 +930,8 @@ default_load_playlists (RBGenericPlayerSource *source)
 		full_playlist_path = rb_uri_append_path (mount_path, playlist_path);
 		rb_debug ("constructed playlist search path %s", full_playlist_path);
 	} else {
-		full_playlist_path = g_strdup (mount_path);
+		g_free (playlist_path);
+		return;
 	}
 
 	/* only try to load playlists if the device has at least one playlist format */
