@@ -54,10 +54,13 @@ static const char *ignored_plugins[] = {
 	"grl-apple-trailers",
 	"grl-bliptv",
 	"grl-bookmarks",
+	"grl-dmap",
 	"grl-filesystem",
 	"grl-flickr",
+	"grl-magnatune",
 	"grl-optical-media",
 	"grl-podcasts",
+	"grl-raitv",
 	"grl-tracker",
 	"grl-vimeo",
 	"grl-youtube"
