@@ -1237,8 +1237,6 @@ load_ipod_db_idle_cb (RBiPodSource *source)
  	GList *it;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 
-	GDK_THREADS_ENTER ();
-
 	db = get_db_for_source (source);
 
 	g_assert (db != NULL);
@@ -1261,7 +1259,6 @@ load_ipod_db_idle_cb (RBiPodSource *source)
 
 	rb_transfer_target_transfer (RB_TRANSFER_TARGET (source), NULL, FALSE);
 
-	GDK_THREADS_LEAVE ();
 	priv->load_idle_id = 0;
 	return FALSE;
 }
