@@ -28,14 +28,13 @@
 import os
 import sys
 import xml
-import urllib
-import urlparse
+import urllib.parse, urllib.request
 import threading
 import zipfile
 
 import rb
 from gi.repository import RB
-from gi.repository import GObject, Gtk, Gdk, Gio
+from gi.repository import GObject, Gtk, Gdk, Gio, GLib
 
 from TrackListHandler import TrackListHandler
 from DownloadAlbumHandler import DownloadAlbumHandler, MagnatuneDownloadError
@@ -65,11 +64,11 @@ class MagnatuneSource(RB.BrowserSource):
 		RB.BrowserSource.__init__(self)
 		self.hate = self
 
+		self.__popup = None
 		self.__settings = Gio.Settings("org.gnome.rhythmbox.plugins.magnatune")
 		# source state
 		self.__activated = False
 		self.__db = None
-		self.__notify_id = 0 # GObject.idle_add id for status notifications
 		self.__info_screen = None # the loading screen
 
 		# track data
@@ -78,12 +77,12 @@ class MagnatuneSource(RB.BrowserSource):
 		self.__art_dict = {}
 
 		# catalogue stuff
-		self.__updating = True # whether we're loading the catalog right now
 		self.__has_loaded = False # whether the catalog has been loaded yet
-		self.__update_id = 0 # GObject.idle_add id for catalog updates
+		self.__update_id = 0 # GLib.idle_add id for catalog updates
 		self.__catalogue_loader = None
 		self.__catalogue_check = None
-		self.__load_progress = (0, 0) # (complete, total)
+		self.__load_progress = None
+		self.__download_progress = None
 
 		# album download stuff
 		self.__downloads = {} # keeps track of download progress for each file
@@ -95,27 +94,16 @@ class MagnatuneSource(RB.BrowserSource):
 	# RBSource methods
 	#
 
-	def do_impl_show_entry_popup(self):
-		self.show_source_popup("/MagnatuneSourceViewPopup")
+	def do_show_entry_popup(self):
+		if self.__popup is None:
+			builder = Gtk.Builder()
+			builder.add_from_file(rb.find_plugin_file(self.props.plugin, "magnatune-popup.ui"))
+			self.__popup = builder.get_object("magnatune-popup")
+
+		menu = Gtk.Menu.new_from_model(self.__popup)
+		menu.attach_to_widget(self, None)
+		menu.popup(None, None, None, None, 3, Gtk.get_current_event_time())
 
-	def do_get_status(self, status, progress_text, progress):
-		if self.__updating:
-			complete, total = self.__load_progress
-			if total > 0:
-				progress = min(float(complete) / total, 1.0)
-			else:
-				progress = -1.0
-			return (_("Loading Magnatune catalog"), None, progress)
-		elif len(self.__downloads) > 0:
-			complete, total = map(sum, zip(*self.__downloads.itervalues()))
-			if total > 0:
-				progress = min(float(complete) / total, 1.0)
-			else:
-				progress = -1.0
-			return (_("Downloading Magnatune Album(s)"), None, progress)
-		else:
-			qm = self.props.query_model
-			return (qm.compute_status_normal("%d song", "%d songs"), None, 2.0)
 
 	def do_selected(self):
 		if not self.__activated:
@@ -125,17 +113,17 @@ class MagnatuneSource(RB.BrowserSource):
 
 			if not magnatune_in_progress_dir.query_exists(None):
 				magnatune_in_progress_path = magnatune_in_progress_dir.get_path()
-				os.mkdir(magnatune_in_progress_path, 0700)
+				os.mkdir(magnatune_in_progress_path, 0o700)
 
 			if not magnatune_cache_dir.query_exists(None):
 				magnatune_cache_path = magnatune_cache_dir.get_path()
-				os.mkdir(magnatune_cache_path, 0700)
+				os.mkdir(magnatune_cache_path, 0o700)
 
 			self.__activated = True
 			self.__show_loading_screen(True)
 
 			# start our catalogue updates
-			self.__update_id = GObject.timeout_add_seconds(6 * 60 * 60, self.__update_catalogue)
+			self.__update_id = GLib.timeout_add_seconds(6 * 60 * 60, self.__update_catalogue)
 			self.__update_catalogue()
 
 	def do_impl_can_delete(self):
@@ -149,13 +137,9 @@ class MagnatuneSource(RB.BrowserSource):
 
 	def do_delete_thyself(self):
 		if self.__update_id != 0:
-			GObject.source_remove(self.__update_id)
+			GLib.source_remove(self.__update_id)
 			self.__update_id = 0
 
-		if self.__notify_id != 0:
-			GObject.source_remove(self.__notify_id)
-			self.__notify_id = 0
-
 		if self.__catalogue_loader is not None:
 			self.__catalogue_loader.cancel()
 			self.__catalogue_loader = None
@@ -189,7 +173,7 @@ class MagnatuneSource(RB.BrowserSource):
 
 		for tr in tracks:
 			sku = self.__sku_dict[tr.get_string(RB.RhythmDBPropType.LOCATION)]
-			url = magnatune_buy_album_uri + urllib.urlencode({ 'sku': sku, 'ref': magnatune_partner_id })
+			url = magnatune_buy_album_uri + urllib.parse.urlencode({ 'sku': sku, 'ref': magnatune_partner_id })
 			if url not in urls:
 				Gtk.show_uri(screen, url, Gdk.CURRENT_TIME)
 				urls.add(url)
@@ -204,7 +188,7 @@ class MagnatuneSource(RB.BrowserSource):
 			# Just use the first library location
 			library = Gio.Settings("org.gnome.rhythmbox.rhythmdb")
 			library_location = library['locations'][0]
-		except IndexError, e:
+		except IndexError as e:
 			RB.error_dialog(title = _("Couldn't download album"),
 				        message = _("You must have a library location set to download an album."))
 			return
@@ -227,20 +211,20 @@ class MagnatuneSource(RB.BrowserSource):
 		def update_cb(remote_changes):
 			self.__catalogue_check = None
 			try:
-				f = open(magnatune_changes, 'r')
+				f = open(magnatune_changes, 'rt')
 				local_changes = f.read().strip()
 			except:
 				local_changes = ""
 
-			remote_changes = remote_changes.strip()
-			print "local checksum %s, remote checksum %s" % (local_changes, remote_changes)
+			remote_changes = remote_changes.strip().decode("iso-8859-1")
+			print("local checksum %s, remote checksum %s" % (local_changes, remote_changes))
 			if local_changes != remote_changes:
 				try:
-					f = open(magnatune_changes, 'w')
+					f = open(magnatune_changes, 'wt')
 					f.write(remote_changes + "\n")
 					f.close()
-				except Exception, e:
-					print "unable to write local change id: %s" % str(e)
+				except Exception as e:
+					print("unable to write local change id: %s" % str(e))
 
 				download_catalogue()
 			elif self.__has_loaded is False:
@@ -254,19 +238,18 @@ class MagnatuneSource(RB.BrowserSource):
 				return None
 
 			def download_progress(copy, complete, total, self):
-				self.__load_progress = (complete, total)
-				self.__notify_status_changed()
+				self.__load_progress.props.task_progress = min(float(complete) / total, 1.0)
 
 			def download_finished(copy, success, self):
 				if not success:
-					print "catalog download failed"
-					print copy.get_error()
+					print("catalog download failed")
+					print(copy.get_error())
 					return
 
-				print "catalog download successful"
+				print("catalog download successful")
 				# done downloading, unzip to real location
 				catalog_zip = zipfile.ZipFile(magnatune_song_info_temp)
-				catalog = open(magnatune_song_info, 'w')
+				catalog = open(magnatune_song_info, 'wb')
 				filename = find_song_info(catalog_zip)
 				if filename is None:
 					RB.error_dialog(title=_("Unable to load catalog"),
@@ -278,41 +261,43 @@ class MagnatuneSource(RB.BrowserSource):
 
 				df = Gio.file_new_for_path(magnatune_song_info_temp)
 				df.delete(None)
-				self.__updating = False
 				self.__catalogue_loader = None
-				self.__notify_status_changed()
+
+				self.__load_progress.props.task_outcome = RB.TaskOutcome.COMPLETE
 
 				load_catalogue()
 
-
-			self.__updating = True
-
 			try:
 				df = Gio.file_new_for_path(magnatune_song_info_temp)
 				df.delete(None)
 			except:
 				pass
+
+			self.__load_progress = RB.TaskProgressSimple.new()
+			self.__load_progress.props.task_label = _("Loading Magnatune catalog")
+			self.props.shell.props.task_list.add_task(self.__load_progress)
+
 			self.__catalog_loader = RB.AsyncCopy()
 			self.__catalog_loader.set_progress(download_progress, self)
 			self.__catalog_loader.start(magnatune_song_info_uri, magnatune_song_info_temp, download_finished, self)
 
 		def load_catalogue():
 
-			def catalogue_chunk_cb(loader, data, total, parser):
-				if data is None:
+			def catalogue_chunk_cb(loader, chunk, total, parser):
+				if chunk is None:
+					self.__load_progress.props.task_outcome = RB.TaskOutcome.COMPLETE
 					error = loader.get_error()
 					if error:
 						# report error somehow?
-						print "error loading catalogue: %s" % error
+						print("error loading catalogue: %s" % error)
 
 					try:
 						parser.close()
-					except xml.sax.SAXParseException, e:
+					except xml.sax.SAXParseException as e:
 						# there isn't much we can do here
-						print "error parsing catalogue: %s" % e
+						print("error parsing catalogue: %s" % e)
 
 					self.__show_loading_screen(False)
-					self.__updating = False
 					self.__catalogue_loader = None
 
 					# restart in-progress downloads
@@ -322,11 +307,13 @@ class MagnatuneSource(RB.BrowserSource):
 						if not name.startswith("in_progress_"):
 							continue
 						(result, uri, etag) = magnatune_in_progress_dir.resolve_relative_path(name).load_contents(None)
-						print "restarting download from %s" % uri
+						uri = uri.decode('utf-8')
+
+						print("restarting download from %s" % uri)
 						self.__download_album(uri, name[12:])
 				else:
 					# hack around some weird chars that show up in the catalogue for some reason
-					data = str(data.str)
+					data = chunk.get_data().decode('utf-8', errors='replace')
 					data = data.replace("\x19", "'")
 					data = data.replace("\x13", "-")
 
@@ -335,19 +322,17 @@ class MagnatuneSource(RB.BrowserSource):
 
 					try:
 						parser.feed(data)
-					except xml.sax.SAXParseException, e:
-						print "error parsing catalogue: %s" % e
+					except xml.sax.SAXParseException as e:
+						print("error parsing catalogue: %s" % e)
 
 					load_size['size'] += len(data)
-					self.__load_progress = (load_size['size'], total)
-
-				self.__notify_status_changed()
+					self.__load_progress.props.task_progress = min(float(load_size['size']) / total, 1.0)
 
 
 			self.__has_loaded = True
-			self.__updating = True
-			self.__load_progress = (0, 0) # (complete, total)
-			self.__notify_status_changed()
+			self.__load_progress = RB.TaskProgressSimple.new()
+			self.__load_progress.props.task_label = _("Loading Magnatune catalog")
+			self.props.shell.props.task_list.add_task(self.__load_progress)
 
 			load_size = {'size': 0}
 
@@ -377,13 +362,7 @@ class MagnatuneSource(RB.BrowserSource):
 		self.__paned_box.set_property("visible", not show)
 
 	def __notify_status_changed(self):
-		def change_idle_cb():
-			self.notify_status_changed()
-			self.__notify_id = 0
-			return False
-
-		if self.__notify_id == 0:
-			self.__notify_id = GObject.idle_add(change_idle_cb)
+		pass
 
 	#
 	# internal purchasing code
@@ -391,7 +370,8 @@ class MagnatuneSource(RB.BrowserSource):
 
 	def __auth_download(self, sku): # http://magnatune.com/info/api
 
-		def auth_data_cb(data, (username, password)):
+		def auth_data_cb(data, userpass):
+			(username, password) = userpass
 			dl_album_handler = DownloadAlbumHandler(self.__settings['format'])
 			auth_parser = xml.sax.make_parser()
 			auth_parser.setContentHandler(dl_album_handler)
@@ -401,6 +381,7 @@ class MagnatuneSource(RB.BrowserSource):
 				return
 
 			try:
+				data = data.decode("utf-8")
 				data = data.replace("<br>", "") # get rid of any stray <br> tags that will mess up the parser
 				data = data.replace(" & ", " &amp; ") # clean up some missing escaping
 				# print data
@@ -408,28 +389,28 @@ class MagnatuneSource(RB.BrowserSource):
 				auth_parser.close()
 
 				# process the URI: add authentication info, quote the filename component for some reason
-				parsed = urlparse.urlparse(dl_album_handler.url)
+				parsed = urllib.parse.urlparse(dl_album_handler.url)
 				netloc = "%s:%s@%s" % (username, password, parsed.hostname)
 
-				spath = os.path.split(urllib.url2pathname(parsed.path))
+				spath = os.path.split(urllib.request.url2pathname(parsed.path))
 				basename = spath[1]
-				path = urllib.pathname2url(os.path.join(spath[0], urllib.quote(basename)))
+				path = urllib.request.pathname2url(os.path.join(spath[0], urllib.parse.quote(basename)))
 
 				authed = (parsed[0], netloc, path) + parsed[3:]
-				audio_dl_uri = urlparse.urlunparse(authed)
+				audio_dl_uri = urllib.parse.urlunparse(authed)
 
-				print "download uri for %s is %s" % (sku, audio_dl_uri)
+				print("download uri for %s is %s" % (sku, audio_dl_uri))
 				self.__download_album(audio_dl_uri, sku)
 
-			except MagnatuneDownloadError, e:
+			except MagnatuneDownloadError as e:
 				RB.error_dialog(title = _("Download Error"),
 						message = _("An error occurred while trying to authorize the download.\nThe Magnatune server returned:\n%s") % str(e))
-			except Exception, e:
+			except Exception as e:
 				sys.excepthook(*sys.exc_info())
 				RB.error_dialog(title = _("Error"),
 						message = _("An error occurred while trying to download the album.\nThe error text is:\n%s") % str(e))
 
-		print "downloading album: " + sku
+		print("downloading album: " + sku)
 		account = MagnatuneAccount.instance()
 		(account_type, username, password) = account.get()
 		url_dict = {
@@ -437,46 +418,48 @@ class MagnatuneSource(RB.BrowserSource):
 			'sku':	sku
 		}
 		url = magnatune_api_download_uri % (username, password)
-		url = url + urllib.urlencode(url_dict)
+		url = url + urllib.parse.urlencode(url_dict)
 
 		l = rb.Loader()
 		l.get_url(url, auth_data_cb, (username, password))
 
 
 	def __download_album(self, audio_dl_uri, sku):
+		def update_progress(self):
+			complete, total = map(sum, zip(*self.__downloads.values()))
+			if total > 0:
+				self.__download_progress.props.task_progress = min(float(complete) / total, 1.0)
+
 		def download_progress(copy, complete, total, self):
 			self.__downloads[audio_dl_uri] = (complete, total)
-			self.__notify_status_changed()
+			update_progress(self)
 
 		def download_finished(copy, success, self):
 			del self.__downloads[audio_dl_uri]
 			del self.__copies[audio_dl_uri]
+			update_progress(self)
 
-			print "download of %s finished: %s" % (audio_dl_uri, success)
+			print("download of %s finished: %s" % (audio_dl_uri, success))
 			if success:
 				threading.Thread(target=unzip_album).start()
 			else:
 				remove_download_files()
 
-			if len(self.__downloads) == 0: # All downloads are complete
-				shell = self.props.shell
-				manager = shell.props.ui_manager
-				manager.get_action("/MagnatuneSourceViewPopup/MagnatuneCancelDownload").set_sensitive(False)
-				if success:
-					shell.notify_custom(4000, _("Finished Downloading"), _("All Magnatune downloads have been completed."), None, False)
+			if len(self.__downloads) == 0:
+				self.__download_progress.props.task_outcome = RB.TaskOutcome.COMPLETE
+				self.__download_progress = None
 
-			self.__notify_status_changed()
 
 		def unzip_album():
 			# just use the first library location
 			library = Gio.Settings("org.gnome.rhythmbox.rhythmdb")
 			library_location = Gio.file_new_for_uri(library['locations'][0])
 
-			print "unzipping %s" % dest.get_path()
+			print("unzipping %s" % dest.get_path())
 			album = zipfile.ZipFile(dest.get_path())
 			for track in album.namelist():
 				track_uri = library_location.resolve_relative_path(track).get_uri()
-				print "zip file entry: %s => %s" % (track, track_uri)
+				print("zip file entry: %s => %s" % (track, track_uri))
 
 				track_uri = RB.sanitize_uri_for_filesystem(track_uri)
 				RB.uri_create_parent_dirs(track_uri)
@@ -485,50 +468,48 @@ class MagnatuneSource(RB.BrowserSource):
 				if track_out is not None:
 					track_out.write(album.read(track), None)
 					track_out.close(None)
-					print "adding %s to library" % track_uri
+					print("adding %s to library" % track_uri)
 					self.__db.add_uri(track_uri)
 
 			album.close()
 			remove_download_files()
 
 		def remove_download_files():
-			print "removing download files"
+			print("removing download files")
 			in_progress.delete(None)
 			dest.delete(None)
 
 		in_progress = magnatune_in_progress_dir.resolve_relative_path("in_progress_" + sku)
 		dest = magnatune_in_progress_dir.resolve_relative_path(sku)
 
-		in_progress.replace_contents(str(audio_dl_uri),
+		in_progress.replace_contents(audio_dl_uri.encode('utf-8'),
 					     None,
 					     False,
 					     Gio.FileCreateFlags.PRIVATE|Gio.FileCreateFlags.REPLACE_DESTINATION,
 					     None)
 
-		shell = self.props.shell
-		manager = shell.props.ui_manager
-		manager.get_action("/MagnatuneSourceViewPopup/MagnatuneCancelDownload").set_sensitive(True)
-
 		try:
 			# For some reason, Gio.FileCopyFlags.OVERWRITE doesn't work for copy_async
 			dest.delete(None)
 		except:
 			pass
 
+		if self.__download_progress is None:
+			self.__download_progress = RB.TaskProgressSimple.new()
+			self.__download_progress.props.task_label = _("Downloading from Magnatune")
+			self.__download_progress.connect('cancel', self.cancel_downloads)
+
 		dl = RB.AsyncCopy()
 		dl.set_progress(download_progress, self)
 		dl.start(audio_dl_uri, dest.get_uri(), download_finished, self)
 		self.__downloads[audio_dl_uri] = (0, 0) # (current, total)
 		self.__copies[audio_dl_uri] = dl
 
-
-	def cancel_downloads(self):
+	def cancel_downloads(self, task):
 		for download in self.__copies.values():
 			download.cancel()
 
-		shell = self.props.shell
-		manager = shell.props.ui_manager
-		manager.get_action("/MagnatuneSourceViewPopup/MagnatuneCancelDownload").set_sensitive(False)
+		task.props.task_outcome = RB.TaskOutcome.CANCELLED
 
 	def playing_entry_changed(self, entry):
 		if not self.__db or not entry:
