@@ -63,6 +63,7 @@ typedef struct
 
 	gchar *notify_art_path;
 	NotifyNotification *notification;
+	NotifyNotification *misc_notification;
 	gboolean notify_supports_actions;
 	gboolean notify_supports_icon_buttons;
 	gboolean notify_supports_persistence;
@@ -172,7 +173,7 @@ do_notify (RBNotificationPlugin *plugin,
 	if (playback) {
 		notification = plugin->notification;
 	} else {
-		notification = NULL;
+		notification = plugin->misc_notification;
 	}
 
 	if (notification == NULL) {
@@ -184,6 +185,8 @@ do_notify (RBNotificationPlugin *plugin,
 					 plugin, 0);
 		if (playback) {
 			plugin->notification = notification;
+		} else {
+			plugin->misc_notification = notification;
 		}
 	} else {
 		notify_notification_clear_hints (notification);
@@ -206,18 +209,24 @@ do_notify (RBNotificationPlugin *plugin,
 
 	notify_notification_clear_actions (notification);
 	if (playback && plugin->notify_supports_actions) {
+		gboolean rtl;
+		const char *play_icon;
+
+		rtl = (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL);
+		play_icon = rtl ? "media-playback-start-rtl" : "media-playback-start";
+
 		if (plugin->notify_supports_icon_buttons) {
 			gboolean playing = FALSE;
 			rb_shell_player_get_playing (plugin->shell_player, &playing, NULL);
 
 			notify_notification_add_action (notification,
-							"media-skip-backward",
+							rtl ? "media-skip-backward-rtl" : "media-skip-backward",
 							_("Previous"),
 							(NotifyActionCallback) notification_previous_cb,
 							plugin,
 							NULL);
 			notify_notification_add_action (notification,
-							playing ? "media-playback-pause" : "media-playback-start",
+							playing ? "media-playback-pause" : play_icon,
 							playing ? _("Pause") : _("Play"),
 							(NotifyActionCallback) notification_playpause_cb,
 							plugin,
@@ -226,7 +235,7 @@ do_notify (RBNotificationPlugin *plugin,
 		}
 
 		notify_notification_add_action (notification,
-						"media-skip-forward",
+						rtl ? "media-skip-forward-rtl" : "media-skip-forward",
 						_("Next"),
 						(NotifyActionCallback) notification_next_cb,
 						plugin,
@@ -282,6 +291,13 @@ cleanup_notification (RBNotificationPlugin *plugin)
 		notify_notification_close (plugin->notification, NULL);
 		plugin->notification = NULL;
 	}
+	if (plugin->misc_notification != NULL) {
+		g_signal_handlers_disconnect_by_func (plugin->misc_notification,
+						      G_CALLBACK (notification_closed_cb),
+						      plugin);
+		notify_notification_close (plugin->misc_notification, NULL);
+		plugin->misc_notification = NULL;
+	}
 }
 
 static void
@@ -489,6 +505,10 @@ playing_entry_changed_cb (RBShellPlayer *player,
 			  RhythmDBEntry *entry,
 			  RBNotificationPlugin *plugin)
 {
+	if (entry == NULL && plugin->notification == NULL) {
+		return;
+	}
+
 	update_current_playing_data (plugin, entry);
 	notify_playing_entry (plugin, FALSE);
 }
@@ -498,6 +518,10 @@ playing_changed_cb (RBShellPlayer *player,
 		    gboolean       playing,
 		    RBNotificationPlugin *plugin)
 {
+	if (playing == FALSE && plugin->notification == NULL) {
+		return;
+	}
+
 	notify_playing_entry (plugin, FALSE);
 }
 
