@@ -242,7 +242,7 @@ about_action_cb (GSimpleAction *action, GVariant *parameters, gpointer user_data
 			       "copyright", "Copyright \xc2\xa9 2005 - 2012 The Rhythmbox authors\nCopyright \xc2\xa9 2003 - 2005 Colin Walters\nCopyright \xc2\xa9 2002, 2003 Jorn Baayen",
 			       "license", license_trans,
 			       "website-label", _("Rhythmbox Website"),
-			       "website", "http://www.gnome.org/projects/rhythmbox",
+			       "website", "https://wiki.gnome.org/Apps/Rhythmbox",
 			       "comments", comment->str,
 			       "authors", (const char **) authors,
 			       "documenters", (const char **) documenters,
