@@ -122,20 +122,9 @@ rb_missing_files_source_class_init (RBMissingFilesSourceClass *klass)
 static void
 rb_missing_files_source_init (RBMissingFilesSource *source)
 {
-	gint size;
-	GdkPixbuf *pixbuf;
-
 	source->priv = G_TYPE_INSTANCE_GET_PRIVATE (source, RB_TYPE_MISSING_FILES_SOURCE, RBMissingFilesSourcePrivate);
 
-	gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
-	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-					   "dialog-warning",
-					   size,
-					   0, NULL);
-	g_object_set (source, "pixbuf", pixbuf, NULL);
-	if (pixbuf != NULL) {
-		g_object_unref (pixbuf);
-	}
+	rb_display_page_set_icon_name (RB_DISPLAY_PAGE (source), "dialog-warning-symbolic");
 }
 
 static void
