@@ -297,9 +297,11 @@ rb_static_playlist_source_constructed (GObject *object)
 	rb_source_search_basic_register (RHYTHMDB_PROP_COMPOSER_FOLDED, "composer", _("Search composers"));
 	rb_source_search_basic_register (RHYTHMDB_PROP_ALBUM_FOLDED, "album", _("Search albums"));
 	rb_source_search_basic_register (RHYTHMDB_PROP_TITLE_FOLDED, "title", _("Search titles"));
+	rb_source_search_basic_register (RHYTHMDB_PROP_GENRE_FOLDED, "genre", _("Search genres"));
 	
 	section = g_menu_new ();
 	rb_source_search_add_to_menu (section, "app", priv->search_action, "search-match");
+	rb_source_search_add_to_menu (section, "app", priv->search_action, "genre");
 	rb_source_search_add_to_menu (section, "app", priv->search_action, "artist");
 	rb_source_search_add_to_menu (section, "app", priv->search_action, "composer");
 	rb_source_search_add_to_menu (section, "app", priv->search_action, "album");
