@@ -230,10 +230,8 @@ buffering_cb (GObject *backend, gpointer whatever, guint progress, RBStreamingSo
 	else if (progress == 100)
 		progress = 0;
 
-	GDK_THREADS_ENTER ();
 	source->priv->buffering = progress;
 	rb_source_notify_playback_status_changed (RB_SOURCE (source));
-	GDK_THREADS_LEAVE ();
 }
 
 static gboolean
