@@ -21,8 +21,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -93,7 +92,7 @@ rb_transfer_target_build_dest_uri (RBTransferTarget *target,
 	if (uri != NULL) {
 		char *sane_uri;
 
-		sane_uri = rb_sanitize_uri_for_filesystem (uri);
+		sane_uri = rb_sanitize_uri_for_filesystem (uri, NULL);
 		g_return_val_if_fail (sane_uri != NULL, NULL);
 		g_free (uri);
 		uri = sane_uri;
