@@ -47,6 +47,8 @@
 #include "rb-file-helpers.h"
 #include "rb-ext-db.h"
 
+#define LABEL_SELECT_PADDING	20
+
 /**
  * SECTION:rb-header
  * @short_description: playback area widgetry
@@ -97,6 +99,7 @@ static void rb_header_sync_time (RBHeader *header);
 static void uri_dropped_cb (RBFadingImage *image, const char *uri, RBHeader *header);
 static void pixbuf_dropped_cb (RBFadingImage *image, GdkPixbuf *pixbuf, RBHeader *header);
 static void image_button_press_cb (GtkWidget *widget, GdkEvent *event, RBHeader *header);
+static gboolean label_button_press_cb (GtkWidget *widget, GdkEventButton *event, RBHeader *header);
 static void art_added_cb (RBExtDB *db, RBExtDBKey *key, const char *filename, GValue *data, RBHeader *header);
 static void volume_widget_changed_cb (GtkScaleButton *widget, gdouble volume, RBHeader *header);
 static void player_volume_changed_cb (RBShellPlayer *player, GParamSpec *pspec, RBHeader *header);
@@ -115,6 +118,7 @@ struct RBHeaderPrivate
 	GtkWidget *songbox;
 	GtkWidget *song;
 	GtkWidget *details;
+	GtkWidget *not_playing;
 	GtkWidget *image;
 	GtkWidget *volume_button;
 
@@ -285,6 +289,7 @@ static void
 rb_header_constructed (GObject *object)
 {
 	RBHeader *header = RB_HEADER (object);
+	char *label;
 
 	RB_CHAIN_GOBJECT_METHOD (rb_header_parent_class, constructed, object);
 
@@ -330,21 +335,45 @@ rb_header_constructed (GObject *object)
 	header->priv->songbox = gtk_grid_new ();
 	gtk_widget_set_hexpand (header->priv->songbox, TRUE);
 	gtk_widget_set_valign (header->priv->songbox, GTK_ALIGN_CENTER);
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (header->priv->songbox), GTK_ORIENTATION_VERTICAL);
 
-	header->priv->song = gtk_label_new (" ");
+	header->priv->song = g_object_ref (gtk_label_new (" "));
+	gtk_widget_show (header->priv->song);
 	gtk_label_set_use_markup (GTK_LABEL (header->priv->song), TRUE);
 	gtk_label_set_selectable (GTK_LABEL (header->priv->song), TRUE);
 	gtk_label_set_ellipsize (GTK_LABEL (header->priv->song), PANGO_ELLIPSIZE_END);
 	gtk_misc_set_alignment (GTK_MISC (header->priv->song), 0.0, 0.5);
-	gtk_grid_attach (GTK_GRID (header->priv->songbox), header->priv->song, 0, 0, 1, 1);
+	g_signal_connect_object (header->priv->song,
+				 "button-press-event",
+				 G_CALLBACK (label_button_press_cb),
+				 header, 0);
 
-	header->priv->details = gtk_label_new ("");
+	header->priv->details = g_object_ref (gtk_label_new (""));
+	gtk_widget_show (header->priv->details);
 	gtk_label_set_use_markup (GTK_LABEL (header->priv->details), TRUE);
 	gtk_label_set_selectable (GTK_LABEL (header->priv->details), TRUE);
 	gtk_label_set_ellipsize (GTK_LABEL (header->priv->details), PANGO_ELLIPSIZE_END);
 	gtk_widget_set_hexpand (header->priv->details, TRUE);
 	gtk_misc_set_alignment (GTK_MISC (header->priv->details), 0.0, 0.5);
-	gtk_grid_attach (GTK_GRID (header->priv->songbox), header->priv->details, 0, 1, 1, 2);
+	g_signal_connect_object (header->priv->details,
+				 "button-press-event",
+				 G_CALLBACK (label_button_press_cb),
+				 header, 0);
+
+	label = g_markup_printf_escaped (TITLE_FORMAT, _("Not Playing"));
+	header->priv->not_playing = g_object_ref (gtk_label_new (label));
+	g_free (label);
+	gtk_widget_show (header->priv->not_playing);
+	gtk_label_set_use_markup (GTK_LABEL (header->priv->not_playing), TRUE);
+	gtk_label_set_selectable (GTK_LABEL (header->priv->not_playing), TRUE);
+	gtk_label_set_ellipsize (GTK_LABEL (header->priv->not_playing), PANGO_ELLIPSIZE_END);
+	gtk_widget_set_hexpand (header->priv->not_playing, TRUE);
+	gtk_misc_set_alignment (GTK_MISC (header->priv->not_playing), 0.0, 0.5);
+	gtk_container_add (GTK_CONTAINER (header->priv->songbox), header->priv->not_playing);
+	g_signal_connect_object (header->priv->not_playing,
+				 "button-press-event",
+				 G_CALLBACK (label_button_press_cb),
+				 header, 0);
 
 	/* elapsed time / duration display */
 	header->priv->timelabel = gtk_label_new ("");
@@ -422,6 +451,10 @@ rb_header_dispose (GObject *object)
 		header->priv->art_store = NULL;
 	}
 
+	g_clear_object (&header->priv->song);
+	g_clear_object (&header->priv->details);
+	g_clear_object (&header->priv->not_playing);
+
 	G_OBJECT_CLASS (rb_header_parent_class)->dispose (object);
 }
 
@@ -787,7 +820,6 @@ get_extra_metadata (RhythmDB *db, RhythmDBEntry *entry, const char *field, char
 static void
 rb_header_sync (RBHeader *header)
 {
-	char *label_text;
 	if (header->priv->entry != NULL) {
 		const char *title;
 		const char *album;
@@ -802,6 +834,12 @@ rb_header_sync (RBHeader *header)
 			  rhythmdb_entry_get_string (header->priv->entry, RHYTHMDB_PROP_LOCATION));
 		gboolean have_duration = (header->priv->duration > 0);
 
+		if (gtk_widget_get_parent (header->priv->song) == NULL) {
+			gtk_container_remove (GTK_CONTAINER (header->priv->songbox), header->priv->not_playing);
+			gtk_container_add (GTK_CONTAINER (header->priv->songbox), header->priv->song);
+			gtk_container_add (GTK_CONTAINER (header->priv->songbox), header->priv->details);
+		}
+
 		title = rhythmdb_entry_get_string (header->priv->entry, RHYTHMDB_PROP_TITLE);
 		album = rhythmdb_entry_get_string (header->priv->entry, RHYTHMDB_PROP_ALBUM);
 		artist = rhythmdb_entry_get_string (header->priv->entry, RHYTHMDB_PROP_ARTIST);
@@ -910,11 +948,11 @@ rb_header_sync (RBHeader *header)
 		g_free (streaming_title);
 	} else {
 		rb_debug ("not playing");
-		label_text = g_markup_printf_escaped (TITLE_FORMAT, _("Not Playing"));
-		gtk_label_set_markup (GTK_LABEL (header->priv->song), label_text);
-		g_free (label_text);
-
-		gtk_label_set_text (GTK_LABEL (header->priv->details), "");
+		if (gtk_widget_get_parent (header->priv->not_playing) == NULL) {
+			gtk_container_remove (GTK_CONTAINER (header->priv->songbox), header->priv->song);
+			gtk_container_remove (GTK_CONTAINER (header->priv->songbox), header->priv->details);
+			gtk_container_add (GTK_CONTAINER (header->priv->songbox), header->priv->not_playing);
+		}
 
 		rb_header_sync_time (header);
 		gtk_widget_set_sensitive (header->priv->scale, FALSE);
@@ -991,14 +1029,9 @@ slider_press_callback (GtkWidget *widget,
 static gboolean
 slider_moved_timeout (RBHeader *header)
 {
-	GDK_THREADS_ENTER ();
-
 	apply_slider_position (header);
 	header->priv->slider_moved_timeout = 0;
 	header->priv->slider_drag_moved = FALSE;
-
-	GDK_THREADS_LEAVE ();
-
 	return FALSE;
 }
 
@@ -1313,3 +1346,61 @@ player_volume_changed_cb (RBShellPlayer *player, GParamSpec *pspec, RBHeader *he
 	gtk_scale_button_set_value (GTK_SCALE_BUTTON (header->priv->volume_button), volume);
 	header->priv->syncing_volume = FALSE;
 }
+
+static gboolean
+do_window_drag (RBHeader *header)
+{
+	GtkWidget *widget;
+	gboolean window_dragging;
+
+	widget = GTK_WIDGET (header);
+	while (widget != NULL) {
+		if (GTK_IS_TOOLBAR (widget)) {
+			gtk_widget_style_get (widget, "window-dragging", &window_dragging, NULL);
+			return window_dragging;
+		}
+
+		widget = gtk_widget_get_parent (widget);
+	}
+
+	return FALSE;
+}
+
+static gboolean
+label_button_press_cb (GtkWidget *label, GdkEventButton *event, RBHeader *header)
+{
+	GtkWidget *window;
+	int min, nat;
+
+	if (do_window_drag (header) == FALSE) {
+		return FALSE;
+	}
+
+	if (gdk_event_triggers_context_menu ((GdkEvent *)event)) {
+		return FALSE;
+	}
+
+	if (event->type != GDK_BUTTON_PRESS) {
+		return FALSE;
+	}
+
+	/* if we're over or near the text, allow the normal selection thing to happen,
+	 * otherwise act like a bit of toolbar
+	 */
+	gtk_widget_get_preferred_width (label, &min, &nat);
+	if (gtk_widget_get_direction (label) == GTK_TEXT_DIR_RTL) {
+		if (event->x > (gtk_widget_get_allocated_width (label) - (nat + LABEL_SELECT_PADDING))) {
+			return FALSE;
+		}
+	} else if (event->x < (nat + LABEL_SELECT_PADDING)) {
+		return FALSE;
+	}
+
+	window = gtk_widget_get_toplevel (label);
+	gtk_window_begin_move_drag (GTK_WINDOW (window),
+				    event->button,
+				    event->x_root,
+				    event->y_root,
+				    event->time);
+	return TRUE;
+}
